package com.zskk.dicom.monitor.schedules;

import com.zskk.dicom.monitor.config.Configs;
import com.zskk.dicom.monitor.runner.*;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 *
 * 功能描述:
 *
 * @param: dirMonitorPools 监控文件线程池 oldUploadPools 旧文件
 * @return:
 * @auther: zzp
 * @date: 2019/3/10 10:35
 */
public class StatRunner {

    /**
     * dcm文件产生文件夹监听线程(池)
     */
    private static ExecutorService dirMonitorPools = null;

    /**
     * 重试文件夹监听线程(池)
     */
    private static ExecutorService retryMonitorPools = null;

    /**
     * 把遗留上传文件添加进入队列的线程(池)
     */
    private static ExecutorService addHistoryFile2QueuePools = null;

    /**
     * 把遗留重试上传文件添加进入队列的线程(池)
     */
    private static ExecutorService addHistoryRetryFile2QueuePools = null;


    /**
     * 上传当前重试文件线程(池)
     */
    private static ExecutorService currentRetryPools = null;
    /**
     * 把文件移动到错误文件的线程(池)
     */
    private static ExecutorService errorPools = null;

    /**
     * 把文件移动到重试文件的线程(池)
     */
    private static ExecutorService remove2RetryPools = null;

    /**
     * 上传遗留上传文件线程池(默认线程数 4 最小线程数 1)
     */
    private static ExecutorService oldUploadPools = null;

    /**
     * 上传当前上传文件线程池(默认线程数 4 最小线程数 1)
     */
    private static ExecutorService currentUploadPools = null;

    /**
     * 上传遗留重试文件线程池(默认线程数 2 最小线程数 1)
     */
    private static ExecutorService oldRetryPools = null;

    /**
     * 把文件移动到备份文件的线程池(默认线程数 2 最小线程数 1)
     */
    private static ExecutorService successPools = null;

    /**
     * 初始化各种线程池并启动线程
     */
    public static void start() {
        // 监听目标文件
        startDirMonitorPools();

        // 监听重试文件
        startRetryMonitorPools();

        //老文件加入老上传队列
        startAddHistoryFile2UploadPools();

        //老重试文件加入老重试上传队列
        startAddHistoryRetryFile2UploadPools();

        //新重试上传
        startCurrentRetryPools();

        //重试移动文件
        startRemove2RetryPools();

        //错误移动文件
        startErrorPools();

        // 成功移动
        startSuccessPools(Math.max(Configs.successPoolsLength, 1));

        // 当前上传
        startCurrentUploadPools(Math.max(Configs.currentUploadPoolsLength, 1));

        try {
            TimeUnit.SECONDS.sleep(1);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        // 老文件上传
        startOldUploadPools(Math.max(Configs.oldUploadPoolsLength, 1));

        // 老重试上传
        startOldRetryPools(Math.max(Configs.oldRetryPoolsLength, 1));
    }

    /**
     * 启动文件移动到错误文件夹的线程池(单线程)
     */
    private static void startErrorPools() {
        errorPools = generateSinglePools(errorPools);
        errorPools.execute(new ErrorRunner());
    }

    /**
     * 启动文件移动到重试文件夹的线程池(单线程)
     */
    private static void startRemove2RetryPools() {
        remove2RetryPools = generateSinglePools(remove2RetryPools);
        remove2RetryPools.execute(new RemoveFile2RetryRunner());
    }

    /**
     * 启动重试上传的线程池(单线程)
     */
    private static void startCurrentRetryPools() {
        currentRetryPools = generateSinglePools(currentRetryPools);
        currentRetryPools.execute(new RetryCurrentUploadRunner());
    }

    /**
     * 启动遍历重试文件夹并把遗留重试文件放入队列的线程池(单线程)
     */
    private static void startAddHistoryRetryFile2UploadPools() {
        addHistoryRetryFile2QueuePools = generateSinglePools(addHistoryRetryFile2QueuePools);
        addHistoryRetryFile2QueuePools.execute(new AddRetryHistory2QueueRunner());
    }

    /**
     * 启动遍历dcm产生文件夹并把遗留上传文件放入上传队列的线程池(单线程)
     */
    private static void startAddHistoryFile2UploadPools() {
        addHistoryFile2QueuePools = generateSinglePools(addHistoryFile2QueuePools);
        addHistoryFile2QueuePools.execute(new AddHistory2QueueRunner());
    }

    /**
     * 启动重试文件夹监听的线程池(单线程)
     */
    private static void startRetryMonitorPools() {
        retryMonitorPools = generateSinglePools(retryMonitorPools);
        retryMonitorPools.execute(new RetryDirMonitor());
    }

    /**
     * 启动目标dcm产生文件夹监听的线程池(单线程)
     */
    private static void startDirMonitorPools() {
        dirMonitorPools = generateSinglePools(dirMonitorPools);
        dirMonitorPools.execute(new DirMonitor());
    }

    /**
     * 启动遗留重试文件上传的线程池
     * @param oldRetryPoolsLength 线程池数量
     */
    private static void startOldRetryPools(int oldRetryPoolsLength) {
        int count = 0;
        oldRetryPools = generatePools(oldRetryPools, oldRetryPoolsLength);
        while (count < oldRetryPoolsLength) {
            oldRetryPools.execute(new RetryHistoryUploadRunner());
            count++;
        }
    }

    /**
     * 启动遗留文件上传的线程池
     * @param oldUploadPoolsLength 线程池数量
     */
    private static void startOldUploadPools(int oldUploadPoolsLength) {
        int count = 0;
        oldUploadPools = generatePools(oldUploadPools, oldUploadPoolsLength);
        while (count < oldUploadPoolsLength) {
            oldUploadPools.execute(new HistoryUploadRunner());
            count++;
        }
    }

    /**
     * 启动当前文件上传的线程池
     * @param currentUploadPoolsLength 线程池数量
     */
    private static void startCurrentUploadPools(int currentUploadPoolsLength) {
        int count = 0;
        currentUploadPools = generatePools(currentUploadPools, currentUploadPoolsLength);
        while (count < currentUploadPoolsLength) {
            currentUploadPools.execute(new CurrentUploadRunner());
            count++;
        }
    }

    /**
     * 启动上传成功后文件移动到备份文件夹的线程池
     * @param successPoolsLength 线程池数量
     */
    private static void startSuccessPools(int successPoolsLength) {
        int count = 0;
        successPools = generatePools(successPools, successPoolsLength);
        while (count < successPoolsLength) {
            successPools.execute(new SuccessRunner());
            count++;
        }
    }

    /**
     * 生成线程池
     * @param oldPool 老线程池
     * @param length 线程数量
     * @return
     */
    public static ExecutorService generatePools(ExecutorService oldPool, int length) {
        if (oldPool != null && !oldPool.isTerminated()) {
            oldPool.shutdown();
            while (oldPool.isTerminated()) {
                try {
                    oldPool.awaitTermination(200, TimeUnit.MILLISECONDS);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
        if(length == 1) {
           return Executors.newSingleThreadExecutor();
        }
        return Executors.newFixedThreadPool(length);
    }

    /**
     * 生成只有一个线程的线程池
     * @param oldPool 老线程池
     * @return
     */
    public static ExecutorService generateSinglePools(ExecutorService oldPool) {
        return generatePools(oldPool, 1);
    }

}
